/***********************
ddrawfunctions.cpp

File contains implementations of drawing functions that
initialize direct draw, draws objects, shutsdown direct draw, etc.
************************/

/***********************
INCLUDES AND VARIABLES
************************/

#include "ddrawfunctions.h"

LPDIRECTDRAW7 dd7object; //general directdraw object
LPDIRECTDRAWSURFACE7 primarysurface; //primary drawing surface
LPDIRECTDRAWSURFACE7 backsurface; //secondary surface (back buffer)
LPDIRECTDRAWCLIPPER dd7clipper;  //directdraw clipper
DDSURFACEDESC2 ddsd; //holds surface description
DDBLTFX ddbltfx;  //holds description of blit effects

/***********************
FUNCTIONS
************************/

/***********************
Initializes direct draw with width, height, color depth and window handle
************************/

int startupDirectDraw(int width, int height, int bpp, HWND handle)
{
  LPDIRECTDRAW  ddtemp = NULL; //plain ddraw object
  
  if(FAILED(DirectDrawCreate(NULL, &ddtemp, NULL)))//creates ddraw object
   if(MessageBox(handle, "Could not create ddtemp", "Error!", MB_OK) == IDOK)
    return -1;

  if(FAILED(ddtemp->QueryInterface(IID_IDirectDraw7, (LPVOID*)&dd7object))) //upgrades to new interface if possible
	  if(MessageBox(handle, "Could not QueryInterface", "Error!", MB_OK) == IDOK)
		  return -1;
	
  //don't need ddtemp anymore
  if(ddtemp)
  {
	  ddtemp->Release();
	  ddtemp = NULL;
  }

  //sets it to fullscreen
  if(FAILED(dd7object->SetCooperativeLevel(handle, DDSCL_ALLOWREBOOT | DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE)))
   if(MessageBox(handle, "Could not SetCooperativeLevel", "Error!", MB_OK) == IDOK)
    return -1;
  
  //set display mode
  if(FAILED(dd7object->SetDisplayMode(width, height, bpp, 0, 0)))
   if(MessageBox(handle, "Could not set display mode", "Error!", MB_OK) == IDOK)
		  return -1;

  //now, create primary surface
  ZeroMemory(&ddsd, sizeof(ddsd)); //clears out ddsd structure
  ddsd.dwSize = sizeof(ddsd);   //ddsd is used to describe surface btw

  ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;

  //means I have 1 backbuffer
  ddsd.dwBackBufferCount = 1;
  
  //want surface to be complex and flippable
  ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_COMPLEX | DDSCAPS_FLIP;
  
  //creates primarysurface surface
  if(FAILED(dd7object->CreateSurface(&ddsd, &primarysurface, NULL)))
   if(MessageBox(handle, "Could not CreateSurface", "Error!", MB_OK) == IDOK)
		  return -1;
  
  ddsd.ddsCaps.dwCaps = DDSCAPS_BACKBUFFER;

  //attach back surface to primary surface so I can do flipmode, haha
  if(FAILED(primarysurface->GetAttachedSurface(&ddsd.ddsCaps, &backsurface))) //attaches back buffer to primary surface
   if(MessageBox(handle, "Could not GetAttachedSurface", "Error!", MB_OK) == IDOK)
		  return -1;  

  
  //create clipper
  if(FAILED(dd7object->CreateClipper(0, &dd7clipper, NULL)))
	  if(MessageBox(handle, "Could not CreateClipper", "Error!", MB_OK) == IDOK)
		  return -1;

  //fullscreen clipper!
  if(FAILED(dd7clipper->SetHWnd(0, handle)))
   if(MessageBox(handle, "Could not SetHWND clipper", "Error!", MB_OK) == IDOK)
		  return -1;

  //attach clipper to surface
  if(FAILED(backsurface->SetClipper(dd7clipper)))
	  if(MessageBox(handle, "Could not SetClipper", "Error!", MB_OK) == IDOK)
		  return -1;

   return 0;

}

/***********************
wrapper that loads up bitmap to a surface and returns the surface
************************/

LPDIRECTDRAWSURFACE7 newPicture(LPCSTR filename)
{
 LPDIRECTDRAWSURFACE7 temp;

 temp = DDLoadBitmap(dd7object, filename, 0, 0);

 return temp;
}

/***********************
draws title screen
************************/

void drawTitle(LPDIRECTDRAWSURFACE7 dest, LPCSTR filename)
{
 LPDIRECTDRAWSURFACE7 titlesurface[4];
 DDCOLORKEY tempkey;
 memset((void*)&tempkey, 0, sizeof(tempkey));

 titlesurface[0] = DDLoadBitmap(dd7object, filename, 0, 0);
 titlesurface[1] = DDLoadBitmap(dd7object, "bitmaps\\background2.bmp", 0, 0);
 titlesurface[2] = DDLoadBitmap(dd7object, "bitmaps\\background3.bmp", 0, 0);
 titlesurface[3] = DDLoadBitmap(dd7object, "bitmaps\\background4.bmp", 0, 0);


 dest->Blt(NULL, titlesurface[0], NULL, DDBLT_WAIT, NULL);
 primarysurface->Flip(dest, DDFLIP_WAIT);
 Sleep(2600);

 dest->Blt(NULL,titlesurface[0], NULL, DDBLT_WAIT, NULL);
 drawObject(300, 500, titlesurface[1], dest, NULL, tempkey);
 primarysurface->Flip(dest, DDFLIP_WAIT);
 Sleep(2600);

 dest->Blt(NULL, titlesurface[2], NULL, DDBLT_WAIT, NULL);
 primarysurface->Flip(dest, DDFLIP_WAIT);
 Sleep(2600);

 dest->Blt(NULL, titlesurface[3], NULL, DDBLT_WAIT, NULL);
 primarysurface->Flip(dest, DDFLIP_WAIT);
 Sleep(2800);

 erase(dest, 0, 0, 0);
 if(FAILED(primarysurface->Flip(NULL, DDFLIP_WAIT)))
  return;

 for(int i = 0; i < 4; i++)
 if(titlesurface[i])
 {
  titlesurface[i]->Release();
  titlesurface[i] = NULL;
 }

}

/***********************
draws pixel of specified color at (x,y) on surface; scales it if scalefactor is nonzero
************************/

int drawPixel(int x, int y, int red, int green, int blue, LPDIRECTDRAWSURFACE7 dest, int scalefactor)
{
 DDBLTFX tempddbltfx; //temporarily holds description of blit effects
 RECT destrect;  //temporarily holds "rectangle" of where to draw pixel

 ZeroMemory(&tempddbltfx, sizeof(tempddbltfx)); //clears out structure
 tempddbltfx.dwSize = sizeof(tempddbltfx); //sets size of structure

 tempddbltfx.dwFillColor = RGB16BIT565(red, green, blue); //sets color of pixel

 //sets up "rectangle" of one pixel; scales if necessary
 destrect.left = x - scalefactor;
 destrect.top = y - scalefactor;
 destrect.right = (x+1) + scalefactor;
 destrect.bottom = (y+1) + scalefactor;

 //blits the pixel to surface
 if(FAILED(dest->Blt(&destrect, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, &tempddbltfx)))
  return -1;

 return 0;

}

/***********************
draws circle of specified radius with center at x, y
************************/

void drawCircle(LPDIRECTDRAWSURFACE7 surface, int x, int y, int radius, int red, int green, int blue, int scalefactor)
{	
 int p, mx, my;
 p = 1 - radius;
 mx = 0;
 my = radius;
 while(mx <= my)
 {
  drawPixel(x+mx,y+my, red, green, blue, surface, scalefactor);
  drawPixel(x+mx,y-my, red, green, blue, surface, scalefactor);
  drawPixel(x-mx,y+my, red, green, blue, surface, scalefactor);
  drawPixel(x-mx,y-my, red, green, blue, surface, scalefactor);
  drawPixel(x+my,y+mx, red, green, blue, surface, scalefactor);
  drawPixel(x+my,y-mx, red, green, blue, surface, scalefactor);
  drawPixel(x-my,y+mx, red, green, blue, surface, scalefactor);
  drawPixel(x-my,y-mx, red, green, blue, surface, scalefactor);
  mx++;

  if(p < 0)
   p += 2 * mx + 1;
  else
  {
   my--;
   p += 2 * (mx-my) + 1;
  }
 }
}


/***********************
draws object at (x,y) on specified surface; scales object if scalefactor is nonzero
************************/

int drawObject(int x, int y, LPDIRECTDRAWSURFACE7 source, LPDIRECTDRAWSURFACE7 dest, int scalefactor, DDCOLORKEY tempkey)
{
 RECT destrect; //temporarily holds rectangle of where to draw object
 DDSURFACEDESC2 tempsd; //temporarily holds description of object surface

 //black is transparent color
 //tempkey.dwColorSpaceLowValue = RGB16BIT565(0, 255, 255);
 //tempkey.dwColorSpaceHighValue = RGB16BIT565(128, 255, 255);

 ZeroMemory(&tempsd, sizeof(tempsd)); //clears out structure
 tempsd.dwSize = sizeof(tempsd);  //sets size of structure

 //sets color key (transparency)
 if(FAILED(source->SetColorKey(DDCKEY_SRCBLT, &tempkey)))
  if(MessageBox(NULL, "setcolorkey didn't work!", "error!", MB_OK) == IDOK)
   return -1;

 //sets up rectangle for drawing
 destrect.left = x - scalefactor;
 destrect.top = y - scalefactor;
 
 //get description so we can find out width and height of bitmap!
 if(FAILED(source->GetSurfaceDesc(&tempsd)))
  if(MessageBox(NULL, "getsurfacedesc didn't work!", "error!", MB_OK) == IDOK)
   return -1;

 //sets up rest up rectange
 destrect.right = (x + (int)tempsd.dwWidth) + scalefactor;
 destrect.bottom = (y + (int)tempsd.dwHeight) + scalefactor;

 //blit object to surface
 if(FAILED(dest->Blt(&destrect, source, NULL, DDBLT_WAIT | DDBLT_KEYSRC, NULL)))
  return -1;

 return 0;
}

/***********************
Uses GDI to draw specified text at location (x,y) to surface
************************/
int drawText(LPDIRECTDRAWSURFACE7 dest, LPCSTR text, int x, int y)
{
 HDC dc;

 if(FAILED(dest->GetDC(&dc)))
  return -1;
 
 SetTextColor(dc, 0x00FFFFFF);
 SetBkMode(dc, TRANSPARENT);

 TextOut(dc, x, y, text, strlen(text));

 if(FAILED(dest->ReleaseDC(dc)))
  return -1;

 return 0;
}




/***********************
erases surface; usually done every frame to "clear out" backbuffer
************************/
 
int erase(LPDIRECTDRAWSURFACE7 surface, int red, int green, int blue)
{
 //if there's no surface, get out of here
 if(NULL == surface)
  return -1;
  
 DDBLTFX ddbltfx;  //temporarily holds description of blit effects

 ZeroMemory( &ddbltfx, sizeof(ddbltfx) );  //clears out structure
 ddbltfx.dwSize      = sizeof(ddbltfx);  //sets size field
 ddbltfx.dwFillColor = RGB16BIT565(red, green, blue);  //fill everything with black

 //fills up surface with black to "erase" surface
 if(FAILED(surface->Blt( NULL, NULL, NULL, DDBLT_COLORFILL, &ddbltfx )))
  return -1;

 return 0;
}

/***********************
releases all direct draw objects created
************************/

void shutdownDirectDraw()
{
 //release clipper object
 if(dd7clipper)
 {
  dd7clipper->Release();
  dd7clipper = NULL;
 }

	//release secondary surface
	if(backsurface)
	{
		backsurface->Release();
		backsurface = NULL;
	}
	
	//release primary surface
	if(primarysurface)
	{
		primarysurface->Release();
		primarysurface = NULL;
	}
    
	//release main directdraw object
	if(dd7object)
	{
		dd7object->Release();
		dd7object = NULL;
	}
}